@@ -417,7 +417,7 @@ DBFOpen( const char * pszFilename, const char * pszAccess )
     psDBF->pszHeader = (char *) pabyBuf;
 
     fseek( psDBF->fp, 32, 0 );
-    if( fread( pabyBuf, nHeadLen-32, 1, psDBF->fp ) != 1 )
+    if( nHeadLen <= 32 || fread( pabyBuf, nHeadLen-32, 1, psDBF->fp ) != 1 )
     {
 	fclose( psDBF->fp );
 	free( pabyBuf );
@@ -443,8 +443,18 @@ DBFOpen( const char * pszFilename, const char * pszAccess )
 	}
 	else
 	{
+	    psDBF->panFieldSize[iField] = pabyFInfo[16];
+	    psDBF->panFieldDecimals[iField] = 0;
+/*
+** The following seemed to be used sometimes to handle files with long
+** string fields, but in other cases (such as bug 1202) the decimals field
+** just seems to indicate some sort of preferred formatting, not very
+** wide fields.  So I have disabled this code.  FrankW.
 	    psDBF->panFieldSize[iField] = pabyFInfo[16] + pabyFInfo[17]*256;
 	    psDBF->panFieldDecimals[iField] = 0;
+*/
+
+/* The above change has been back-ported from shapelib per PR#15248 */
 	}
 
 	psDBF->pachFieldType[iField] = (char) pabyFInfo[11];
@@ -535,8 +545,9 @@ DBFCreate( const char * pszFilename )
     if( pszBasename[i] == '.' )
 	pszBasename[i] = '\0';
 
-    pszFullname = (char *) malloc(strlen(pszBasename) + 5);
-    sprintf( pszFullname, "%s.dbf", pszBasename );
+    int sz = strlen(pszBasename) + 5;
+    pszFullname = (char *) malloc(sz);
+    snprintf( pszFullname, sz, "%s.dbf", pszBasename );
     free( pszBasename );
 
 /* -------------------------------------------------------------------- */
@@ -1091,8 +1102,8 @@ static int DBFWriteAttribute(DBFHandle psDBF, int hEntity, int iField,
 	    if( sizeof(szSField)-2 < nWidth )
 		nWidth = sizeof(szSField)-2;
 
-	    sprintf( szFormat, "%%%dd", nWidth );
-	    sprintf(szSField, szFormat, (int) *((double *) pValue) );
+	    snprintf( szFormat, 20, "%%%dd", nWidth );
+	    snprintf(szSField, 400, szFormat, (int) *((double *) pValue) );
 	    if( (int)strlen(szSField) > psDBF->panFieldSize[iField] )
 	    {
 		szSField[psDBF->panFieldSize[iField]] = '\0';
@@ -1108,9 +1119,9 @@ static int DBFWriteAttribute(DBFHandle psDBF, int hEntity, int iField,
 	    if( sizeof(szSField)-2 < nWidth )
 		nWidth = sizeof(szSField)-2;
 
-	    sprintf( szFormat, "%%%d.%df",
+	    snprintf( szFormat, 20, "%%%d.%df",
 		     nWidth, psDBF->panFieldDecimals[iField] );
-	    sprintf(szSField, szFormat, *((double *) pValue) );
+	    snprintf(szSField, 400, szFormat, *((double *) pValue) );
 	    if( (int) strlen(szSField) > psDBF->panFieldSize[iField] )
 	    {
 		szSField[psDBF->panFieldSize[iField]] = '\0';
